@@ -25,21 +25,80 @@ type Position = i64;
 type GroupIDs = HashSet<GroupId>;
 type RecordIDs = Vec<RecordId>;
 
-#[derive(Hash, PartialEq, Eq)]
+#[derive(Hash, PartialEq, Eq, Debug)]
 pub enum RecordId {
     Regular(Vec<u8>),
     Split(Vec<u8>),
 }
 
-#[derive(Hash, PartialEq, Eq, Clone)]
+#[derive(Hash, PartialEq, Eq, Clone, Debug)]
 pub enum GroupId {
     Regular(u32),
     Split(u32),
 }
 
+#[derive(new, Debug)]
+pub struct GroupEndIndex {
+    #[new(default)]
+    group_pos: HashMap<GroupId, Position>,
+    #[new(default)]
+    group_end_idx: BTreeMap<Position, GroupIDs>,
+}
+
+impl GroupEndIndex {
+    ///Inserts a new group id at given position
+    ///If position is already saved for the group id the group-end-index will be updated
+    pub fn insert(&mut self, group_id: GroupId, end_pos: i64) -> Result<()> {
+        let update_end_pos = match self.group_pos.get(&group_id) {
+            Some(&current_end_pos) => match current_end_pos < end_pos {
+                true => {
+                    self.group_end_idx
+                        .get_mut(&current_end_pos)
+                        .map(|group_ids| group_ids.remove(&group_id));
+                    true
+                }
+                false => false,
+            },
+            None => true,
+        };
+        if update_end_pos {
+            self.group_pos.insert(group_id.clone(), end_pos);
+            self.group_end_idx
+                .entry(end_pos)
+                .or_insert_with(HashSet::new)
+                .insert(group_id);
+        }
+        Ok(())
+    }
+
+    pub fn cut_lower_group_ids(&mut self, current_pos: Option<i64>) -> Result<Vec<GroupId>> {
+        let group_ids: Vec<GroupId> = self
+            .group_end_idx
+            .range(
+                ..current_pos.unwrap_or(
+                    self.group_end_idx
+                        .iter()
+                        .next_back()
+                        .map_or(0, |(entry, _)| *entry)
+                        + 1,
+                ),
+            )
+            .flat_map(|(_, group_ids)| group_ids.clone())
+            .collect();
+        group_ids.iter().for_each(|group_id| {
+            self.group_pos.remove(group_id);
+        });
+        match current_pos {
+            Some(pos) => self.group_end_idx = self.group_end_idx.split_off(&pos),
+            None => self.group_end_idx.clear(),
+        }
+        Ok(group_ids)
+    }
+}
+
 impl<W: io::Write> CallConsensusRead<W> {
     pub fn call_consensus_reads(&mut self) -> Result<()> {
-        let mut group_end_idx: BTreeMap<Position, GroupIDs> = BTreeMap::new();
+        let mut group_end_idx = GroupEndIndex::new();
         let mut duplicate_groups: HashMap<GroupId, RecordIDs> = HashMap::new();
         let mut record_storage: HashMap<RecordId, RecordStorage> = HashMap::new();
         let mut current_chrom = None;
@@ -68,11 +127,6 @@ impl<W: io::Write> CallConsensusRead<W> {
                     &mut self.bam_skipped_writer,
                     &mut read_ids,
                 )?;
-                if record_pos.is_some() {
-                    group_end_idx = group_end_idx.split_off(&record.pos());
-                } else {
-                    group_end_idx.clear();
-                }
             }
             if record.is_unmapped() || record.is_mate_unmapped() {
                 self.bam_skipped_writer.write(&record)?;
@@ -86,42 +140,99 @@ impl<W: io::Write> CallConsensusRead<W> {
             let duplicate_id_option = match record.aux(b"DI") {
                 Ok(Aux::I8(duplicate_id)) => Some(duplicate_id as u32),
                 Ok(Aux::I16(duplicate_id)) => Some(duplicate_id as u32),
+                Ok(Aux::I32(duplicate_id)) => Some(duplicate_id as u32),
                 Ok(Aux::U8(duplicate_id)) => Some(duplicate_id as u32),
                 Ok(Aux::U16(duplicate_id)) => Some(duplicate_id as u32),
                 Ok(Aux::U32(duplicate_id)) => Some(duplicate_id),
                 Err(_) => None,
                 _ => unreachable!("Invalid type for tag 'DI'"),
             };
-            let record_id = record.qname();
-            read_ids.as_mut().map(|x| x.insert(i, record_id.to_vec()));
+            let record_name = record.qname();
+            read_ids.as_mut().map(|x| x.insert(i, record_name.to_vec()));
             //Check if record has duplicate ID
             match duplicate_id_option {
                 //Case: duplicate ID exists
                 Some(duplicate_id) => {
-                    match record_storage.get_mut(&RecordId::Regular(record_id.to_owned())) {
+                    let regular_id = RecordId::Regular(record_name.to_owned());
+                    let record_end_pos = record.cigar_cached().unwrap().end_pos() - 1;
+                    match record_storage.get_mut(&regular_id) {
                         //Case: Right record
                         Some(storage_entry) => {
                             //For right record save end position and duplicate group ID
-                            let record_end_pos = record.pos() + record.seq_len() as i64 - 1;
-                            let group_id = match storage_entry {
-                                RecordStorage::PairedRecords { ref mut r2_rec, .. } => {
-                                    let group_id = duplicate_id;
-                                    r2_rec.get_or_insert(IndexedRecord {
-                                        rec: record,
-                                        rec_id: i,
-                                    });
-                                    GroupId::Regular(group_id)
+                            let group_id_opt = match storage_entry {
+                                RecordStorage::PairedRecords {
+                                    r1_rec,
+                                    ref mut r2_rec,
+                                } => {
+                                    let group_id = if cigar_has_softclips(r1_rec)
+                                        || cigar_has_softclips(&record)
+                                    {
+                                        self.bam_skipped_writer.write(r1_rec)?;
+                                        self.bam_skipped_writer.write(&record)?;
+                                        None
+                                    } else {
+                                        duplicate_groups
+                                            .entry(GroupId::Regular(duplicate_id))
+                                            .or_insert_with(Vec::new)
+                                            .push(RecordId::Regular(record_name.to_owned()));
+                                        r2_rec.get_or_insert(IndexedRecord {
+                                            rec: record,
+                                            rec_id: i,
+                                        });
+                                        Some(GroupId::Regular(duplicate_id))
+                                    };
+                                    group_id
                                 }
                                 // This arm is reached if a mate is mapped to another chromosome.
                                 // In that case a new duplicate and record ID is required
-                                RecordStorage::SingleRecord { .. } => {
-                                    let group_id = duplicate_id;
+                                RecordStorage::SingleRecord { rec } => {
+                                    let group_id = if cigar_has_softclips(rec)
+                                        || cigar_has_softclips(&record)
+                                    {
+                                        self.bam_skipped_writer.write(rec)?;
+                                        self.bam_skipped_writer.write(&record)?;
+                                        None
+                                    } else {
+                                        duplicate_groups
+                                            .entry(GroupId::Split(duplicate_id))
+                                            .or_insert_with(Vec::new)
+                                            .push(RecordId::Split(record_name.to_owned()));
+                                        record_storage.insert(
+                                            RecordId::Split(record_name.to_owned()),
+                                            RecordStorage::SingleRecord {
+                                                rec: IndexedRecord {
+                                                    rec: record,
+                                                    rec_id: i,
+                                                },
+                                            },
+                                        );
+                                        Some(GroupId::Split(duplicate_id))
+                                    };
+                                    group_id
+                                }
+                            };
+                            if let Some(group_id) = group_id_opt {
+                                group_end_idx.insert(group_id, record_end_pos)?;
+                            } else {
+                                record_storage.remove(&regular_id);
+                            };
+                        }
+                        //Case: Left record or record w/o mate
+                        None => {
+                            if !record.is_paired() {
+                                //If right or single record save end position and duplicate group ID
+                                if cigar_has_softclips(&record) {
+                                    self.bam_skipped_writer.write(&record)?;
+                                } else {
                                     duplicate_groups
-                                        .entry(GroupId::Split(group_id))
+                                        .entry(GroupId::Regular(duplicate_id))
                                         .or_insert_with(Vec::new)
-                                        .push(RecordId::Split(record_id.to_owned()));
+                                        .push(RecordId::Regular(record_name.to_owned()));
+
+                                    group_end_idx
+                                        .insert(GroupId::Regular(duplicate_id), record_end_pos)?;
                                     record_storage.insert(
-                                        RecordId::Split(record_id.to_owned()),
+                                        RecordId::Regular(record_name.to_owned()),
                                         RecordStorage::SingleRecord {
                                             rec: IndexedRecord {
                                                 rec: record,
@@ -129,39 +240,10 @@ impl<W: io::Write> CallConsensusRead<W> {
                                             },
                                         },
                                     );
-                                    GroupId::Split(group_id)
                                 }
-                            };
-                            group_end_idx
-                                .entry(record_end_pos)
-                                .or_insert_with(HashSet::new)
-                                .insert(group_id);
-                        }
-                        //Case: Left record or record w/o mate
-                        None => {
-                            duplicate_groups
-                                .entry(GroupId::Regular(duplicate_id))
-                                .or_insert_with(Vec::new)
-                                .push(RecordId::Regular(record_id.to_owned()));
-                            if !record.is_paired() {
-                                //If right or single record save end position and duplicate group ID
-                                let record_end_pos = record.pos() + record.seq_len() as i64 - 1;
-                                group_end_idx
-                                    .entry(record_end_pos)
-                                    .or_insert_with(HashSet::new)
-                                    .insert(GroupId::Regular(duplicate_id));
-                                record_storage.insert(
-                                    RecordId::Regular(record_id.to_owned()),
-                                    RecordStorage::SingleRecord {
-                                        rec: IndexedRecord {
-                                            rec: record,
-                                            rec_id: i,
-                                        },
-                                    },
-                                );
                             } else {
                                 record_storage.insert(
-                                    RecordId::Regular(record_id.to_owned()),
+                                    RecordId::Regular(record_name.to_owned()),
                                     RecordStorage::PairedRecords {
                                         r1_rec: IndexedRecord {
                                             rec: record,
@@ -179,14 +261,14 @@ impl<W: io::Write> CallConsensusRead<W> {
                 //If record is right mate consensus is calculated
                 //Else record is added to hashMap
                 None => {
-                    match record_storage.get_mut(&RecordId::Regular(record_id.to_owned())) {
+                    match record_storage.get_mut(&RecordId::Regular(record_name.to_owned())) {
                         //Case: Left record
                         None => {
                             if !record.is_paired() || record.tid() != record.mtid() {
                                 self.bam_skipped_writer.write(&record)?;
                             } else {
                                 record_storage.insert(
-                                    RecordId::Regular(record_id.to_owned()),
+                                    RecordId::Regular(record_name.to_owned()),
                                     RecordStorage::PairedRecords {
                                         r1_rec: IndexedRecord {
                                             rec: record,
@@ -200,7 +282,7 @@ impl<W: io::Write> CallConsensusRead<W> {
                         //Case: Left record already stored
                         Some(_record_pair) => {
                             let (rec_id, l_rec) = match record_storage
-                                .remove(&RecordId::Regular(record_id.to_owned()))
+                                .remove(&RecordId::Regular(record_name.to_owned()))
                                 .unwrap()
                             {
                                 RecordStorage::PairedRecords { r1_rec, .. } => {
@@ -260,7 +342,7 @@ impl<W: io::Write> CallConsensusRead<W> {
 
 #[allow(clippy::too_many_arguments)]
 pub fn calc_consensus_complete_groups<'a, W: io::Write>(
-    group_end_idx: &mut BTreeMap<Position, GroupIDs>,
+    group_end_idx: &mut GroupEndIndex,
     duplicate_groups: &mut HashMap<GroupId, RecordIDs>,
     end_pos: Option<i64>,
     record_storage: &mut HashMap<RecordId, RecordStorage>,
@@ -270,25 +352,10 @@ pub fn calc_consensus_complete_groups<'a, W: io::Write>(
     bam_skipped_writer: &'a mut bam::Writer,
     read_ids: &'a mut Option<HashMap<usize, Vec<u8>>>,
 ) -> Result<()> {
-    let group_ids: HashSet<GroupId> = group_end_idx
-        .range(
-            ..end_pos.unwrap_or(
-                group_end_idx
-                    .iter()
-                    .next_back()
-                    .map_or(0, |(entry, _)| *entry)
-                    + 1,
-            ),
-        )
-        .flat_map(|(_, group_ids)| group_ids.clone())
-        .collect();
+    let group_ids = group_end_idx.cut_lower_group_ids(end_pos)?;
     for group_id in group_ids {
-        let cigar_groups = group_reads_by_cigar(
-            duplicate_groups.remove(&group_id).unwrap(),
-            record_storage,
-            bam_skipped_writer,
-        )?;
-
+        let cigar_groups =
+            group_reads_by_cigar(duplicate_groups.remove(&group_id).unwrap(), record_storage)?;
         for cigar_group in cigar_groups.values() {
             match cigar_group {
                 CigarGroup::PairedRecords {
@@ -364,81 +431,11 @@ pub fn calc_consensus_complete_groups<'a, W: io::Write>(
 fn group_reads_by_cigar(
     record_ids: Vec<RecordId>,
     record_storage: &mut HashMap<RecordId, RecordStorage>,
-    bam_skipped_writer: &mut bam::Writer,
 ) -> Result<HashMap<Cigar, CigarGroup>> {
     let mut cigar_groups: HashMap<Cigar, CigarGroup> = HashMap::new();
     for rec_id in record_ids {
-        match record_storage.remove(&rec_id).unwrap() {
-            RecordStorage::PairedRecords { r1_rec, r2_rec } => {
-                let r1_rec_id = r1_rec.rec_id;
-                let r1_rec_entry = r1_rec.into_rec();
-                let r2_rec_unwrapped = r2_rec.unwrap();
-                let r2_rec_id = r2_rec_unwrapped.rec_id;
-                let r2_rec_entry = r2_rec_unwrapped.into_rec();
-
-                if cigar_has_softclips(&r1_rec_entry) || cigar_has_softclips(&r2_rec_entry) {
-                    bam_skipped_writer.write(&r1_rec_entry)?;
-                    bam_skipped_writer.write(&r2_rec_entry)?;
-                } else {
-                    let cigar_tuple = Cigar::Tuple {
-                        r1_cigar: r1_rec_entry.raw_cigar().to_vec(),
-                        r2_cigar: r2_rec_entry.raw_cigar().to_vec(),
-                    };
-                    if !cigar_groups.contains_key(&cigar_tuple) {
-                        cigar_groups.insert(
-                            cigar_tuple.clone(),
-                            CigarGroup::PairedRecords {
-                                r1_recs: Vec::new(),
-                                r2_recs: Vec::new(),
-                                r1_seqids: Vec::new(),
-                                r2_seqids: Vec::new(),
-                            },
-                        );
-                    }
-                    match cigar_groups.get_mut(&cigar_tuple) {
-                        Some(CigarGroup::PairedRecords {
-                            r1_recs,
-                            r2_recs,
-                            r1_seqids,
-                            r2_seqids,
-                        }) => {
-                            r1_recs.push(r1_rec_entry);
-                            r2_recs.push(r2_rec_entry);
-                            r1_seqids.push(r1_rec_id);
-                            r2_seqids.push(r2_rec_id);
-                        }
-                        _ => unreachable!(),
-                    }
-                }
-            }
-            RecordStorage::SingleRecord { rec } => {
-                let rec_id = rec.rec_id;
-                let rec_entry = rec.into_rec();
-                if cigar_has_softclips(&rec_entry) {
-                    bam_skipped_writer.write(&rec_entry)?;
-                } else {
-                    let cigar_single = Cigar::Single {
-                        cigar: rec_entry.raw_cigar().to_vec(),
-                    };
-                    if !cigar_groups.contains_key(&cigar_single) {
-                        cigar_groups.insert(
-                            cigar_single.clone(),
-                            CigarGroup::SingleRecords {
-                                recs: Vec::new(),
-                                seqids: Vec::new(),
-                            },
-                        );
-                    }
-                    match cigar_groups.get_mut(&cigar_single) {
-                        Some(CigarGroup::SingleRecords { recs, seqids }) => {
-                            recs.push(rec_entry);
-                            seqids.push(rec_id);
-                        }
-                        _ => unreachable!(),
-                    }
-                }
-            }
-        };
+        let storage_entry = record_storage.remove(&rec_id).unwrap();
+        storage_entry.add_to_group(&mut cigar_groups)?;
     }
     Ok(cigar_groups)
 }
@@ -569,6 +566,7 @@ fn match_single_cigar(cigar: &Option<char>, first_vec: &mut Vec<bool>, second_ve
         Some(_) | None => {}
     };
 }
+
 pub enum RecordStorage {
     PairedRecords {
         r1_rec: IndexedRecord,
@@ -579,6 +577,79 @@ pub enum RecordStorage {
     },
 }
 
+impl RecordStorage {
+    fn add_to_group(self, cigar_groups: &mut HashMap<Cigar, CigarGroup>) -> Result<()> {
+        let (r1_rec_entry, r1_rec_id, r2_rec_entry, r2_rec_id, cigar_tuple) = match self {
+            RecordStorage::PairedRecords { r1_rec, r2_rec } => {
+                let r1_rec_id = r1_rec.rec_id;
+                let r1_rec_entry = r1_rec.into_rec();
+                let r2_rec_unwrapped = r2_rec.unwrap();
+                let r2_rec_id = r2_rec_unwrapped.rec_id;
+                let r2_rec_entry = r2_rec_unwrapped.into_rec();
+                let cigar_tuple = Cigar::Tuple {
+                    r1_cigar: r1_rec_entry.raw_cigar().to_vec(),
+                    r2_cigar: r2_rec_entry.raw_cigar().to_vec(),
+                };
+                if !cigar_groups.contains_key(&cigar_tuple) {
+                    cigar_groups.insert(
+                        cigar_tuple.clone(),
+                        CigarGroup::PairedRecords {
+                            r1_recs: Vec::new(),
+                            r2_recs: Vec::new(),
+                            r1_seqids: Vec::new(),
+                            r2_seqids: Vec::new(),
+                        },
+                    );
+                }
+                (
+                    r1_rec_entry,
+                    r1_rec_id,
+                    Some(r2_rec_entry),
+                    Some(r2_rec_id),
+                    cigar_tuple,
+                )
+            }
+            RecordStorage::SingleRecord { rec } => {
+                let rec_id = rec.rec_id;
+                let rec_entry = rec.into_rec();
+                let cigar_single = Cigar::Single {
+                    cigar: rec_entry.raw_cigar().to_vec(),
+                };
+                if !cigar_groups.contains_key(&cigar_single) {
+                    cigar_groups.insert(
+                        cigar_single.clone(),
+                        CigarGroup::SingleRecords {
+                            recs: Vec::new(),
+                            seqids: Vec::new(),
+                        },
+                    );
+                }
+                (rec_entry, rec_id, None, None, cigar_single)
+            }
+        };
+        match cigar_groups.get_mut(&cigar_tuple) {
+            Some(CigarGroup::PairedRecords {
+                r1_recs,
+                r2_recs,
+                r1_seqids,
+                r2_seqids,
+            }) => {
+                r1_recs.push(r1_rec_entry);
+                r2_recs.push(r2_rec_entry.unwrap());
+                r1_seqids.push(r1_rec_id);
+                r2_seqids.push(r2_rec_id.unwrap());
+            }
+            Some(CigarGroup::SingleRecords { recs, seqids }) => {
+                recs.push(r1_rec_entry);
+                seqids.push(r1_rec_id);
+            }
+            None => unreachable!(),
+        }
+
+        Ok(())
+    }
+}
+
 pub struct IndexedRecord {
     rec: bam::Record,
     rec_id: usize,
